/**
 * Copyright 2017 Jose Nelson Amaral
 * Copyright 2017 Marcus Karpoff
 * 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 * 
 * 1. Redistributions of source code must retain the above copyright notice,
 * this list of conditions and the following disclaimer.
 * 
 * 2. Redistributions in binary form must reproduce the above copyright notice,
 * this list of conditions and the following disclaimer in the documentation
 * and/or other materials provided with the distribution.
 * 
 * 3. Neither the name of the copyright holder nor the names of its
 * contributors may be used to endorse or promote products derived from this
 * software without specific prior written permission.
 * 
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE
 * LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
 * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
 * SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
 * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
 * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
 * POSSIBILITY OF SUCH DAMAGE.
 */
#ifndef _A_STAR_H_
#define _A_STAR_H_

#include <map>

#include "mesh.h"
#include "route.h"
#include "node.h"

class AStarSearcher {
	public:
		AStarSearcher(const Mesh & m): mesh_(m) {};
		
		int cost(const Route & src, const Route & tgt);
		int cost(const Node & src, const Node & tgt, int (Mesh::*cost_func)(Edge) const);
		int path(std::vector<Node> & route, const Node & src, const Node & tgt);
		int path(std::vector<Node> & route, const Node & src, const Node & tgt, int (Mesh::*cost_func)(Edge)const);
		bool has_path_without(Node src, Node tgt, Node exclusion);


	private:
		const Mesh & mesh_;
		std::map<int, std::map<int,int>> cost_mem_;
		int a_star_cost(Node src, Node tgt);
		int cotos(int x, int y, int bl);
		Node stoco(int c, int bl);
		int heuristic(const Node & src, const Node & tgt);
};
#endif
