/**
 * Copyright 2017 Jose Nelson Amaral
 * Copyright 2017 Marcus Karpoff
 * 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 * 
 * 1. Redistributions of source code must retain the above copyright notice,
 * this list of conditions and the following disclaimer.
 * 
 * 2. Redistributions in binary form must reproduce the above copyright notice,
 * this list of conditions and the following disclaimer in the documentation
 * and/or other materials provided with the distribution.
 * 
 * 3. Neither the name of the copyright holder nor the names of its
 * contributors may be used to endorse or promote products derived from this
 * software without specific prior written permission.
 * 
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE
 * LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
 * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
 * SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
 * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
 * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
 * POSSIBILITY OF SUCH DAMAGE.
 */
#ifndef _CITY_H_
#define _CITY_H_

#include "node.h"
#include "mesh.h"
#include "route.h"
#include "route_table.h"
#include "a_star.h"

#include <iostream>
#include <set>
#include <vector>

extern int MAX_START_TIME;
extern int MAX_DEAD_HEADS;
extern float WAIT_COST;
extern float TRAVEL_COST;
extern std::string ROUTE_METHOD;
extern std::string TIME_METHOD;

class City {
	public:
		City(int edge_size);
		~City();
		void add_random_cost_bloom(int num_blooms);
		void add_random_diagonal_path(int num_diagonals);
		void remove_extra_nodes( int desired_node_count );
		void remove_extra_edges( int desired_edge_count );
		void create_routes( int desired_route_count, int desired_runs_per_route);
		void write_test(std::ostream& out);
		void print_bus_map(std::ostream& out) const;


	private:
		Mesh mesh_;
		AStarSearcher searcher;
		RouteTable routes_;
		
		void add_diagonal_path( Node center );
		void add_directional_path( Node start, int x_change, int y_change);
		void select_nodes_alg( std::set<Node>& dest, int num_nodes);
		void select_edges_alg( std::set<Edge>& dest, int num_edges );
		void select_nodes_random( std::set<Node>& dest, int num_nodes);
		void select_edges_random( std::set<Edge>& dest, int num_edges);
		void create_routes_alg( std::vector<Route>& dest, int num_routes);
		void create_start_time_alg( std::vector<int>& dest, int num_times);
		void create_routes_random( std::vector<Route>& dest, int num_routes);
		void create_routes_radial( std::vector<Route>& dest, int num_routes);
		void create_start_time_random( std::vector<int>& dest, int num_times);
		void create_start_time_circadian( std::vector<int>& dest, int num_times);

		void add_route_cost( Route & route );
		void move_node( Node& n, int dir );
	#ifdef NUM_THREADS
		void count_costs( int * c, int thread_id, int* stats);
	#endif

		friend std::ostream& operator<<(std::ostream& out, City const& c);
};

std::ostream& operator<<(std::ostream& out, City const& c);

#endif
