/**
 * Copyright 2017 Jose Nelson Amaral
 * Copyright 2017 Marcus Karpoff
 * 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 * 
 * 1. Redistributions of source code must retain the above copyright notice,
 * this list of conditions and the following disclaimer.
 * 
 * 2. Redistributions in binary form must reproduce the above copyright notice,
 * this list of conditions and the following disclaimer in the documentation
 * and/or other materials provided with the distribution.
 * 
 * 3. Neither the name of the copyright holder nor the names of its
 * contributors may be used to endorse or promote products derived from this
 * software without specific prior written permission.
 * 
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE
 * LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
 * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
 * SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
 * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
 * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
 * POSSIBILITY OF SUCH DAMAGE.
 */
#include "edge.h"

#include <cassert>

Edge::Edge(const Node start, const Node end)
{
	assert(start != end);
	assert(abs(start.x - end.x) <= 1 and abs(start.y - end.y) <= 1);
	if (start < end) {
		this->start = start;
		this->end = end;
	} else {
		this->start = end;
		this->end = start;
	}
}

bool operator< (const Edge& t, const Edge& o)
{
	if (t.start == o.start) {
		return t.end < o.end;
	}	return t.start < o.start;
}
bool operator==(const Edge& t, const Edge& o)
{ return t.start == o.start and t.end == o.end; }
bool operator!=(const Edge& t, const Edge& o)
{ return !(t == o); }

std::ostream& operator<<(std::ostream& out, Edge const& e)
{
	return out << "{" << e.start << ',' << ')';
}
