/**
 * Copyright 2017 Jose Nelson Amaral
 * Copyright 2017 Marcus Karpoff
 * 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 * 
 * 1. Redistributions of source code must retain the above copyright notice,
 * this list of conditions and the following disclaimer.
 * 
 * 2. Redistributions in binary form must reproduce the above copyright notice,
 * this list of conditions and the following disclaimer in the documentation
 * and/or other materials provided with the distribution.
 * 
 * 3. Neither the name of the copyright holder nor the names of its
 * contributors may be used to endorse or promote products derived from this
 * software without specific prior written permission.
 * 
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE
 * LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
 * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
 * SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
 * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
 * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
 * POSSIBILITY OF SUCH DAMAGE.
 */
#include "route.h"


Route::Route()
{
	r_num_ = rand();
}

int get_direction(const Node& s, const Node& e);

std::ostream& operator<<(std::ostream& out, Route const& r)
{
	out << "Run_time: " << r.run_time << std::endl;
	for (auto t: r.start_time) {
		out << t << ' ';
	}
	out << std::endl;
	auto l = r.path.cbegin();
	out << *l << ' ';
	for (auto n = ++r.path.cbegin(); n != r.path.cend(); ++ n) {
		switch (get_direction(*l, *n)) {
		case 0:
			out << u8"\u2191"; break;
		case 1:
			out << u8"\u2191"<< u8"\u2192"; break;
		case 2:
			out << u8"\u2192"; break;
		case 3:
			out << u8"\u2193" << u8"\u2192"; break;
		case 4:
			out << u8"\u2193"; break;
		case 5:
			out << u8"\u2193" << u8"\u2190"; break;
		case 6:
			out << u8"\u2190"; break;
		case 7:
			out << u8"\u2191" << u8"\u2190"; break;
		}
		out << ' ' << *n << ' ';
		l = n;
	}
	return out;
}

int get_direction(const Node& s, const Node& e)
{
	switch(e.x - s.x) {
	case 1:
		return 2 + (e.y - s.y);
	case 0:
		return 2 + 2 * (e.y - s.y);
	case -1:
		return 6 - (e.y - s.y);
	}
	return -1;
}

bool operator==(const Route & t, const Route & o)
{
	return t.r_num_ == o.r_num_;
}
